"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.Logger = exports.LogLevels = void 0;

var _indexOf = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/index-of"));

var _keys = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/keys"));

var _stringify = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/json/stringify"));

var _concat = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/concat"));

var _trim = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/trim"));

var _forEach = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/for-each"));

var _includes = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/includes"));

var _filter = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/filter"));

var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/map"));

var _isArray = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/array/is-array"));

const async = require('async');

require('colors');

const fs = require('fs-extra');

const date = require('date-and-time');

const {
  escapeHtml
} = require('./utils');

const util = require('util');

const DATE_FORMAT = 'YYYY-MM-DD HH:mm:ss:SSS';
const DELIM = '  ';
const LogLevels = {
  error: 'error',
  warn: 'warn',
  info: 'info',
  debug: 'debug',
  trace: 'trace'
};
exports.LogLevels = LogLevels;

class Logger {
  constructor(options) {
    var _context2;

    this.events = options.events || {
      emit: function () {}
    };
    this.logLevel = options.logLevel || 'info';
    this._logFunction = options.logFunction || console.log;
    this.fs = options.fs || fs;

    this.logFunction = function (args, color) {
      var _context;

      args = (0, _isArray.default)(args) ? args : [args];

      this._logFunction(...(0, _map.default)(_context = (0, _filter.default)(args).call(args, arg => arg !== null && arg !== void 0 ? arg : false)).call(_context, arg => {
        if (typeof arg === 'object') arg = util.inspect(arg, 2);
        return color ? arg[color] : arg;
      }));
    };

    this.logFile = options.logFile;

    if (this.logFile) {
      this.fs.ensureFileSync(this.logFile);
    }

    const isDebugOrTrace = (0, _includes.default)(_context2 = ['debug', 'trace']).call(_context2, this.logLevel);
    this.isDebugOrTrace = isDebugOrTrace;

    const noop = () => {};

    this.writeToFile = async.cargo((tasks, callback = noop) => {
      if (!this.logFile) {
        return callback();
      }

      let logs = '';
      (0, _forEach.default)(tasks).call(tasks, task => {
        var _context3, _context4;

        let message = (0, _trim.default)(_context3 = (0, _concat.default)(_context4 = []).call(_context4, task.args).join(' ')).call(_context3);
        if (!message) return;
        const dts = `[${date.format(new Date(), DATE_FORMAT)}]`;
        message = message.replace(/\s+/g, ' ');
        let origin = '';
        if (isDebugOrTrace) origin = `${DELIM}${task.origin.match(/^at\s+.*(\(.*\))/)[1] || '(unknown)'}`;
        const prefix = task.prefix;
        logs += `${dts}${DELIM}${prefix}${DELIM}${message}${origin}\n`;
      });

      if (!logs) {
        callback();
      }

      this.fs.appendFile(this.logFile, logs.stripColors, err => {
        if (err) {
          this.logFunction(`There was an error writing to the log file: ${err}`, 'red');
          return callback(err);
        }

        callback();
      });
    });
  }

  registerAPICall(plugins) {
    let plugin = plugins.createPlugin('logger', {});
    plugin.registerAPICall('ws', '/embark-api/logs', (ws, _req) => {
      this.events.on("log", (logLevel, logMsg) => {
        logMsg = escapeHtml(logMsg);
        ws.send((0, _stringify.default)({
          msg: logMsg,
          msg_clear: logMsg.stripColors,
          logLevel: logLevel
        }), () => {});
      });
    });
  }

  error(...args) {
    if (!args.length || !this.shouldLog('error')) {
      return;
    }

    this.events.emit("log", "error", args);
    this.logFunction(args, 'red');
    let origin;

    if (this.isDebugOrTrace) {
      try {
        var _context5;

        const stack = new Error().stack;
        origin = (0, _trim.default)(_context5 = stack.split('\n')[2]).call(_context5); // eslint-disable-next-line no-empty
      } catch (e) {}
    }

    this.writeToFile.push({
      args,
      origin,
      prefix: "[error]"
    });
  }

  warn(...args) {
    if (!args.length || !this.shouldLog('warn')) {
      return;
    }

    this.events.emit("log", "warn", args);
    this.logFunction(args, 'yellow');
    let origin;

    if (this.isDebugOrTrace) {
      try {
        var _context6;

        const stack = new Error().stack;
        origin = (0, _trim.default)(_context6 = stack.split('\n')[2]).call(_context6); // eslint-disable-next-line no-empty
      } catch (e) {}
    }

    this.writeToFile.push({
      args,
      origin,
      prefix: "[warn]"
    });
  }

  info(...args) {
    if (!args.length || !this.shouldLog('info')) {
      return;
    }

    this.events.emit("log", "info", args);
    this.logFunction(args, 'green');
    let origin;

    if (this.isDebugOrTrace) {
      try {
        var _context7;

        const stack = new Error().stack;
        origin = (0, _trim.default)(_context7 = stack.split('\n')[2]).call(_context7); // eslint-disable-next-line no-empty
      } catch (e) {}
    }

    this.writeToFile.push({
      args,
      origin,
      prefix: "[info]"
    });
  }

  consoleOnly(...args) {
    if (!args.length || !this.shouldLog('info')) {
      return;
    }

    this.logFunction(args, 'green');
    let origin;

    if (this.isDebugOrTrace) {
      try {
        var _context8;

        const stack = new Error().stack;
        origin = (0, _trim.default)(_context8 = stack.split('\n')[2]).call(_context8); // eslint-disable-next-line no-empty
      } catch (e) {}
    }

    this.writeToFile.push({
      args,
      origin,
      prefix: "[consoleOnly]"
    });
  }

  debug(...args) {
    if (!args.length || !this.shouldLog('debug')) {
      return;
    }

    this.events.emit("log", "debug", args);
    this.logFunction(args, null);
    let origin;

    if (this.isDebugOrTrace) {
      try {
        var _context9;

        const stack = new Error().stack;
        origin = (0, _trim.default)(_context9 = stack.split('\n')[2]).call(_context9); // eslint-disable-next-line no-empty
      } catch (e) {}
    }

    this.writeToFile.push({
      args,
      origin,
      prefix: "[debug]"
    });
  }

  trace(...args) {
    if (!args.length || !this.shouldLog('trace')) {
      return;
    }

    this.events.emit("log", "trace", args);
    this.logFunction(args, null);
    let origin;

    if (this.isDebugOrTrace) {
      try {
        var _context10;

        const stack = new Error().stack;
        origin = (0, _trim.default)(_context10 = stack.split('\n')[2]).call(_context10); // eslint-disable-next-line no-empty
      } catch (e) {}
    }

    this.writeToFile.push({
      args,
      origin,
      prefix: "[trace]"
    });
  }

  dir(obj) {
    if (!obj || !this.shouldLog('info')) {
      return;
    }

    this.events.emit("log", "dir", obj);
    this.logFunction(obj, null);
  }

  shouldLog(level) {
    const logLevels = (0, _keys.default)(LogLevels);
    return (0, _indexOf.default)(logLevels).call(logLevels, level) <= (0, _indexOf.default)(logLevels).call(logLevels, this.logLevel);
  }

}

exports.Logger = Logger;
//# sourceMappingURL=index.js.map